<?php

declare(strict_types=1);

namespace Annotation;

use Doctrine\Common\Annotations\Reader;
use Symfony\Component\Routing\Annotation\Route as AnnotationRoute;
use Symfony\Component\Routing\Loader\AnnotationClassLoader as AnnotationClassLoaderSF;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Class AnnotationClassLoaderI18n
 * @package Annotation
 */
final class AnnotationClassLoaderI18n extends AnnotationClassLoaderSF
{
    /**
     * @var \LocalizationInterface
     */
    private $localization;

    /**
     * AnnotationClassLoaderI18n constructor.
     *
     * @param \LocalizationInterface $localization
     */
    public function __construct(
        \LocalizationInterface $localization,
        Reader $reader
    ) {
        parent::__construct($reader);
        $this->localization = $localization;
    }

    /**
     * @param Route             $route
     * @param \ReflectionClass  $class
     * @param \ReflectionMethod $method
     * @param Route             $annot
     */
    protected function configureRoute(
        Route $route,
        \ReflectionClass $class,
        \ReflectionMethod $method,
        $annot
    ): void {
        $route->setDefault('_controller', $class->getName());
        $route->setDefault('_action', $method->getName());
        $route->setPath('/{_locale}' . $route->getPath());
    }

    /**
     * @param RouteCollection                             $collection
     * @param AnnotationRoute                             $annot
     * @param                                             $globals
     * @param \ReflectionClass                            $class
     * @param \ReflectionMethod                           $method
     */
    protected function addRoute(
        RouteCollection $collection,
        $annot,
        $globals,
        \ReflectionClass $class,
        \ReflectionMethod $method
    ): void {
        $defaults = $annot->getDefaults();
        $requireLang = $this->localization->getLocales();

        if (!empty($defaults['locales'])) {
            foreach ($requireLang as $locale) {
                $annotLocale = $this->i18nAnnot($annot, $defaults, $locale);

                parent::addRoute(
                    $collection,
                    $annotLocale,
                    $globals,
                    $class,
                    $method
                );
            }
        } else {
            $requirements = $annot->getRequirements();
            $requirements += ["_locale" => implode('|', $requireLang)];
            $annot->setRequirements($requirements);

            parent::addRoute($collection, $annot, $globals, $class, $method);
        }
    }

    /**
     * @param       $annot
     * @param array $defaults
     * @param       $locale
     *
     * @return mixed
     */
    private function i18nAnnot(AnnotationRoute $annot, array $defaults, string $locale): AnnotationRoute
    {
        $annotLocale = clone $annot;
        if (!empty($defaults['locales'][$locale])) {
            $annotLocale->setPath($defaults['locales'][$locale]);
        }
        $annotLocale->setName($annot->getName() . '_' . $locale);
        $requirements = $annot->getRequirements() + ["_locale" => $locale];
        $annotLocale->setRequirements($requirements);

        return $annotLocale;
    }
}
