<?php

use Annotation\AnnotationClassLoaderI18n;
use Doctrine\Common\Annotations\AnnotationException;
use Doctrine\Common\Annotations\AnnotationReader;
use Doctrine\Common\Annotations\CachedReader;
use Doctrine\Common\Cache\ArrayCache;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\Config\Loader\LoaderResolverInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Routing\Exception\InvalidParameterException;
use Symfony\Component\Routing\Loader\AnnotationClassLoader;
use Symfony\Component\Routing\Loader\AnnotationDirectoryLoader;
use Symfony\Component\Routing\RouteCollection;

/**
 * Class RouterTest
 */
class RouterTest extends TestCase
{
    public function generateDataProvider()
    {
        return [
            [
                'route_name' => 'no_i18n_route',
                'locale' => 'fr',
                'generate_params' => [],
                'url_expected' => '/fr/no_i18n_route.html',
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route',
                'locale' => 'fr',
                'generate_params' => [],
                'url_expected' => '/fr/route_avec_i18n.html',
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => '404_route',
                'locale' => 'fr',
                'generate_params' => [],
                'url_expected' => '',
                'exception_expected' => RouteAndI18nRouteNotFoundException::class,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route_and_with_requirements',
                'locale' => 'fr',
                'generate_params' => ['slug' => 'valid_slug'],
                'url_expected' => "/fr/route_avec_i18n_et_condition/valid_slug",
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route_and_with_requirements',
                'locale' => 'fr',
                'generate_params' => ['slug' => '|invalid_slug'],
                'url_expected' => "",
                'exception_expected' => InvalidParameterException::class,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'no_i18n_route',
                'locale' => 'de',
                'generate_params' => [],
                'url_expected' => '/de/no_i18n_route.html',
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route',
                'locale' => 'de',
                'generate_params' => [],
                'url_expected' => '/de/with_i18n_route.html',
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => '404_route',
                'locale' => 'de',
                'generate_params' => [],
                'url_expected' => '',
                'exception_expected' => RouteAndI18nRouteNotFoundException::class,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route_and_with_requirements',
                'locale' => 'de',
                'generate_params' => ['slug' => 'valid_slug'],
                'url_expected' => "/de/with_i18n_route_and_with_requirements/valid_slug",
                'exception_expected' => null,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_i18n_route_and_with_requirements',
                'locale' => 'de',
                'generate_params' => ['slug' => '|invalid_slug'],
                'url_expected' => "",
                'exception_expected' => InvalidParameterException::class,
                'resource' => MockController::class,
            ],
            [
                'route_name' => 'with_prefix_no_i18n_route',
                'locale' => 'fr',
                'generate_params' => [],
                'url_expected' => "/fr/prefix/no_i18n_route.html",
                'exception_expected' => null,
                'resource' => MockWithPrefixController::class,
            ],
            [
                'route_name' => 'with_prefix_with_i18n_route',
                'locale' => 'fr',
                'generate_params' => [],
                'url_expected' => "/fr/prefix/route_avec_i18n.html",
                'exception_expected' => null,
                'resource' => MockWithPrefixController::class,
            ],
        ];
    }

    /**
     * @dataProvider generateDataProvider
     *
     * @param       $routeName
     * @param       $locale
     * @param array $generateUrlParams
     * @param       $urlExpected
     * @param       $exceptionExcpected
     * @param       $resource
     *
     * @throws RuntimeException
     * @throws AnnotationException
     * @throws ReflectionException
     */
    public function testItGeneratesCorrectUrls(
        $routeName,
        $locale,
        array $generateUrlParams,
        $urlExpected,
        $exceptionExcpected,
        $resource
    ) {
        $cache = new ArrayCache();
        $reader = new CachedReader(
            new AnnotationReader(),
            $cache,
            $debug = true
        );
        $langueLocalization = $this
            ->getMockBuilder(LocalizationInterface::class)
            ->setMethods(['getLocales'])
            ->getMock()
        ;
        $langueLocalization
            ->method('getLocales')
            ->willReturn(['fr', 'en', 'de'])
        ;
        $annotationClassLoader = new AnnotationClassLoaderI18n(
            $langueLocalization,
            $reader
        );
        $router = new Router(
            $annotationClassLoader,
            new MockLoader($annotationClassLoader),
            $resource
        );
        $router->getContext()->setParameter('_locale', $locale);

        if ($exceptionExcpected) {
            $this->expectException($exceptionExcpected);
        }

        $url = $router->generate($routeName, $generateUrlParams);
        $this->assertEquals($urlExpected, $url);

        if ($exceptionExcpected) {
            // pour debug
            dump($url);

            return;
        }
    }
}


class MockController
{
    /**
     * @Route(
     *     "/no_i18n_route.html",
     *     name="no_i18n_route"
     * )
     */
    public function no_i18n_route()
    {

    }

    /**
     * @Route(
     *     "/with_i18n_route.html",
     *     name="with_i18n_route",
     *     defaults={
     *          "locales" = {
     *              "fr": "/route_avec_i18n.html",
     *              "en": "/with_i18n_route.html"
     *          }
     *     }
     * )
     */
    public function with_i18n_route()
    {

    }

    /**
     * @Route(
     *      "/with_i18n_route_and_with_requirements/{slug}",
     *      name="with_i18n_route_and_with_requirements",
     *      defaults={
     *          "locales" = {
     *              "fr": "/route_avec_i18n_et_condition/{slug}",
     *              "en": "/with_i18n_route_and_with_requirements/{slug}"
     *          }
     *      },
     *      requirements = {
     *          "slug": "[a-zA-Z0-9-_,]+"
     *      }
     * )
     */
    public function with_i18n_route_and_with_requirements()
    {

    }
}

/**
 * Class MockWithPrefixController
 *
 * @Route("/prefix")
 */
class MockWithPrefixController
{
    /**
     * @Route(
     *     "/no_i18n_route.html",
     *     name="with_prefix_no_i18n_route"
     * )
     */
    public function no_i18n_route()
    {

    }

    /**
     * @Route(
     *     "/with_i18n_route.html",
     *     name="with_prefix_with_i18n_route",
     *     defaults={
     *          "locales" = {
     *              "fr": "/route_avec_i18n.html",
     *              "en": "/with_i18n_route.html"
     *          }
     *     }
     * )
     */
    public function with_i18n_route()
    {

    }
}

class MockLoader extends AnnotationDirectoryLoader implements LoaderInterface
{
    protected $resolver;

    /**
     * @var AnnotationClassLoader
     */
    protected $annotationClassLoader;

    public function __construct(
        AnnotationClassLoader $annotationClassLoader
    ) {
        $this->annotationClassLoader = $annotationClassLoader;
    }

    /**
     * Loads a resource.
     *
     * @param RouteCollection $resource The resource
     * @param string|null     $type     The resource type or null if unknown
     *
     * @return mixed|RouteCollection
     * @throws \Exception If something went wrong
     */
    public function load($resource, $type = null)
    {
        return $this->annotationClassLoader->load($resource);
    }

    /**
     * Returns whether this class supports the given resource.
     *
     * @param mixed $resource   A resource
     * @param string|null $type The resource type or null if unknown
     *
     * @return bool True if this class supports the given resource, false
     *              otherwise
     */
    public function supports($resource, $type = null)
    {
        return class_exists($resource);
    }

    /**
     * {@inheritdoc}
     */
    public function getResolver()
    {
        return $this->resolver;
    }

    /**
     * {@inheritdoc}
     */
    public function setResolver(LoaderResolverInterface $resolver)
    {
        $this->resolver = $resolver;
    }
}